 package org.netkernel.module.standard;

import org.netkernel.container.IKernel;
import org.netkernel.layer0.meta.IPrototypeMeta;
import org.netkernel.layer0.meta.IPrototypeParameterMeta;
import org.netkernel.layer0.meta.ISpaceElements;
import org.netkernel.layer0.meta.impl.SimpleSpaceElementsImpl;
import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFRequestReadOnly;
import org.netkernel.layer0.nkf.INKFResolutionContext;
import org.netkernel.layer0.nkf.INKFResponseReadOnly;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.nkf.impl.NKFLifecycleEventContextImpl;
import org.netkernel.layer0.util.PairList;
import org.netkernel.layer0.util.Utils;
import org.netkernel.layer0.util.XMLUtils;
import org.netkernel.module.standard.endpoint.IStandardEndpoint;
import org.netkernel.module.standard.endpoint.StandardEndpointImpl;
import org.netkernel.urii.IMetaRepresentation;
import org.netkernel.urii.INetKernelException;
import org.netkernel.urii.IResolution;
import org.netkernel.urii.ISpace;
import org.w3c.dom.Element;


public class LateBoundPrototype extends StandardEndpointImpl implements IStandardEndpoint, ILateBound
{
	private PairList mParameters;
	private boolean mBindAttempted;
	
	public static final String PARAM_PROTO="prototype";
	
	public LateBoundPrototype()
	{	mParameters = new PairList(10);
	}
	
	public void setParameter(String aParameterName, Element aParameterElement, IKernel aKernel) throws NKFException
	{	mParameters.put(aParameterName, aParameterElement);
	}
	public void setParameterRaw(String aParameterName, Object aValue, IKernel aKernel) throws NKFException
	{	mParameters.put(aParameterName, aValue);
	}
	
	private Element getConfigurationElement()
	{	return (Element)mParameters.get(PARAM_PROTO);
	}	
	
	public IStandardEndpoint bind(NKFLifecycleEventContextImpl aContext) throws Exception
	{
		IStandardEndpoint result;
		mBindAttempted=true;
		try
		{
			String prototypeId=XMLUtils.getText(getConfigurationElement());
			
			INKFRequest req=aContext.createRequest(prototypeId);
			req.setVerb(INKFRequestReadOnly.VERB_META);
			req.setRepresentationClass(IMetaRepresentation.class);
			INKFResponseReadOnly resp;
			try
			{	resp = aContext.issueRequestForResponse(req);
			}
			catch (NKFException e)
			{	throw generateParseException("MSG_STD_PROTO_NOT_FOUND",prototypeId,e);
			}
			
			Object rep = resp.getRepresentation();
			if (rep instanceof IPrototypeMeta)
			{	IPrototypeMeta protoMeta = (IPrototypeMeta)rep;
				Class implementationClass=protoMeta.getImplementation();
				//test to ensure we have access to implementationClass
				String implClassName=implementationClass.getName();
				try
				{	getSpace().getClassLoader().loadClass(implClassName);
				}
				catch (Exception e)
				{	throw SpaceFactory.generateParseException("MSG_STD_EP_CLASS_NOT_FOUND",getSpace().toString(),e.getMessage(),getKernel());
				}
				
				Object o=implementationClass.newInstance();
				if (! (o instanceof IStandardEndpoint))
				{	throw generateParseException("MSG_STD_EP_NOT_COMPATIBLE",implementationClass.getName());
				}
				
				result = (IStandardEndpoint)o;
				result.declareParameters(protoMeta.getParameters());
				
				for (int i=0; i<mParameters.size(); i++)
				{	String paramName=(String)mParameters.getValue1(i);
					if (!paramName.equals(PARAM_PROTO))
					{	Object value=mParameters.getValue2(i);
						if (value instanceof Element)
						{	Element paramNode = (Element)value;
							result.setParameter(paramName, paramNode,getKernel());
						}
						else
						{	result.setParameterRaw(paramName, value,getKernel());
						}
					}
				}
				
				//now see if result itself needs further binding
				if (result instanceof ILateBound)
				{	ILateBound lb = (ILateBound)result;
					if (!lb.isBound())
					{	lb.bind(aContext);
					}
				}
			}
			else
			{	throw generateParseException("MSG_STD_PROTO_NOT_FOUND",prototypeId);
			}
		}
		catch (Exception e)
		{	this.setInitialisationProblem(e.getMessage());
			throw e;
		}
		
		return result;
	}
	
	private INetKernelException generateParseException(String aMessageId, Object aParam)
	{	return generateParseException(aMessageId, aParam,null);
	}
	
	
	private INetKernelException generateParseException(String aMessageId, Object aParam, Throwable aCause)
	{	String ownerName;
		if (getSpace() instanceof StandardSpace)
		{	ownerName= ((StandardSpace)getSpace()).getOwnerName();
		}
		else
		{	ownerName=getSpace().toString();
		}
		return Utils.createFormattedException("EX_STD_CONSTRUCT", aMessageId,  getKernel().getLogger(), aCause, ownerName, aParam);
	}

	public ISpaceElements getParts(INKFRequestContext aContext) throws NKFException
	{	return new SimpleSpaceElementsImpl(null,this);
	}
	

	public ISpace[] getSpaces(INKFRequestContext context) throws Exception
	{	return IStandardEndpoint.NO_SPACES;
	}	

	public void onResolve(INKFResolutionContext aContext) throws NKFException
	{
	}

	public boolean isThreadSafe()
	{	// doesn't matter as this will never be used anyway
		return false;
	}

	public void onCommissionEndpoint(IKernel aKernel, ISpace aSpace) throws Exception
	{	super.onCommissionEndpoint(aKernel, aSpace);
	}
	
	public String toString()
	{
		String prototypeId=XMLUtils.getText(getConfigurationElement());
		return "Unresolved prototype ["+prototypeId+"]";
	}

	public void declareParameters(IPrototypeParameterMeta[] aParameterDefn)
	{
	}

	public boolean isBound()
	{	return mBindAttempted;
	}

}