package org.netkernel.module.standard.builtin.branchmerge;

import org.netkernel.container.IKernel;
import org.netkernel.module.standard.ILateBound;
import org.netkernel.module.standard.endpoint.IStandardEndpoint;
import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestReadOnly;
import org.netkernel.layer0.nkf.INKFResolutionContext;
import org.netkernel.layer0.nkf.impl.NKFLifecycleEventContextImpl;
import org.netkernel.layer0.nkf.impl.NKFSpaceImpl;
import org.netkernel.layer0.nkf.impl.ParametersImpl;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.request.impl.RequestScopeLevelImpl;
import org.netkernel.urii.ClassLoaderWithExports;
import org.netkernel.urii.ISpace;

public class BranchMergeSpace extends NKFSpaceImpl
{
	private IStandardEndpoint mOverlay;
	private ISpace mDelegateSpace;
	private ISpace mNextSpace;
	private String mName;
	private BranchMergeOverlay mOwner;
	
	public BranchMergeSpace(IStandardEndpoint aOverlay, ISpace aDelegateSpace, ISpace aNextSpace, String aName, BranchMergeOverlay aOwner)
	{	mOverlay=aOverlay;
		mDelegateSpace=aDelegateSpace;
		mNextSpace=aNextSpace;
		mName=aName;
		mOwner=aOwner;
	}
	
	public void onCommissionSpace(IKernel aKernel) throws Exception
	{
		super.onCommissionSpace(aKernel);
		mOverlay.onCommissionEndpoint(aKernel, this);
	}
	
	public void postCommissionSpace() throws Exception
	{	super.postCommissionSpace();
	
		if (mOverlay instanceof ILateBound)
		{	ILateBound lb = (ILateBound)mOverlay;
			IKernel k=getKernel();
			IRequestScopeLevel scope=RequestScopeLevelImpl.createRootScopeLevel(this);
			NKFLifecycleEventContextImpl context = new NKFLifecycleEventContextImpl(k,scope,this,ParametersImpl.EMPTY);
			mOverlay=lb.bind(context);
			mOverlay.onCommissionEndpoint(k, this);
		}
	
		IRequestScopeLevel scope = RequestScopeLevelImpl.createRootScopeLevel(this);
		mOverlay.postCommissionMapping(scope);
	}
	
	public void preDecommissionSpace() throws Exception
	{	IRequestScopeLevel scope = RequestScopeLevelImpl.createRootScopeLevel(this);
		mOverlay.preDecommissionMapping(scope);
		super.preDecommissionSpace();
	}	
	
	public void onDecommissionSpace() throws Exception
	{	mOverlay.onDecommissionEndpoint();
		super.onDecommissionSpace();
	}
	
	public void onResolve(INKFResolutionContext aContext) throws Exception
	{	
		if (aContext.getRequestToResolve().getVerb()==INKFRequestReadOnly.VERB_META)
		{	aContext.delegateResolution(mDelegateSpace);
		}
		else
		{	mOverlay.onResolve(aContext);
			if (!aContext.isResponseSet())
			{	boolean ready=!(mOverlay instanceof ILateBound) || ((ILateBound)mOverlay).isBound();
				if (!ready)
				{	aContext.delegateResolution(mDelegateSpace);
				}
			}
		}
	}
	
	public String getName()
	{	return mName;
	}
	
	public ISpace getDelegateSpace()
	{	return mDelegateSpace;
	}
	
	public ISpace getNextSpace()
	{	return mNextSpace;
	}
	
	public String toString()
	{	return "BranchMergeSpace["+mName+"]";
	}
	
	public ClassLoaderWithExports getClassLoader()
	{	return mDelegateSpace.getClassLoader();
	}	
}